library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;

package signal_processing_p is

    type real_array_t is array (natural range <>) of real;

end package signal_processing_p;


package body signal_processing_p is



end package body;